@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2019-2021 Kevin Hendricks, Doug Massay
+**  Copyright (C) 2019-2023 Kevin Hendricks, Doug Massay
 **
 **  This file is part of PageEdit.
 **
@@ -52,6 +52,8 @@
 #include "Inspector.h"
 #include "SettingsStore.h"
 #include "Utility.h"
+#include "ClipEditor.h"
+#include "ClipsWindow.h"
 #include "WebViewEdit.h"
 #include "SelectCharacter.h"
 #include "SelectFiles.h"
@@ -122,9 +124,21 @@ MainWindow::MainWindow(QString filepath, QString spineno, QWidget *parent)
     m_skipPrintWarnings(false),
     m_skipPrintPreview(false),
     m_WebViewPrinter(new WebViewPrinter(this)),
+    m_Clips(nullptr),
+    m_ClipEditor(new ClipEditor(this)),
     m_LastPtr(-1)
 {
     ui.setupUi(this);
+    // initialize the toolbar UI clip actions
+    foreach(QAction* clipaction, ui.toolBarClips->actions()) {
+        if (!clipaction->isSeparator()) {
+            QString strIndex = clipaction->objectName();
+            strIndex.replace(QString("actionClip"), QString(""));
+            clipaction->setData(strIndex.toInt());
+            m_clactions.append(clipaction);
+        }
+    }
+    UpdateClipsUI();
     SetupView();
     // start up in edit mode unless the user changes it
     ui.actionMode->setChecked(true);
@@ -132,6 +146,7 @@ MainWindow::MainWindow(QString filepath, QString spineno, QWidget *parent)
     ConnectSignalsToSlots();
     SetupFileList(filepath, spineno);
     SetupNavigationComboBox();
+    m_ClipEditor->SetCSSList(m_CSSList);
     QTimer::singleShot(200, this, SLOT(DoUpdatePage()));
 }
 
@@ -208,6 +223,7 @@ void MainWindow::SetupFileList(const QString &filepath, const QString &spineno)
             media_list << filepath;
             m_MediaKind << "svgimage";
         }
+        m_CSSList = opfrdr.GetCSSFilePathList();
         m_MediaBase = Utility::longestCommonPath(media_list, "/");
         m_MediaList.clear();
         foreach(QString mf, media_list) {
@@ -259,10 +275,11 @@ void MainWindow::CBNavigateActivated(int index)
     }
     
     if ((index > -1) && (index != m_ListPtr)) { 
-        AllowSaveIfModified();
-        m_ListPtr = index;
-        m_CurrentFilePath = m_Base + m_SpineList.at(m_ListPtr);
-        UpdatePage(m_CurrentFilePath);
+        if (AllowSaveIfModified()) {
+            m_ListPtr = index;
+            m_CurrentFilePath = m_Base + m_SpineList.at(m_ListPtr);
+            UpdatePage(m_CurrentFilePath);
+        }
     }
 }
 
@@ -271,12 +288,13 @@ void MainWindow::EditNext()
     if (m_UpdatePageInProgress) return;
     int n = m_SpineList.length();
     if (n > 1) {
-        AllowSaveIfModified();
-        m_ListPtr++;
-        if (m_ListPtr >= n) m_ListPtr = 0;
-        m_CurrentFilePath = m_Base + m_SpineList.at(m_ListPtr);
-        ui.cbNavigate->setCurrentIndex(m_ListPtr);
-        UpdatePage(m_CurrentFilePath);
+        if (AllowSaveIfModified()) {
+            m_ListPtr++;
+            if (m_ListPtr >= n) m_ListPtr = 0;
+            m_CurrentFilePath = m_Base + m_SpineList.at(m_ListPtr);
+            ui.cbNavigate->setCurrentIndex(m_ListPtr);
+            UpdatePage(m_CurrentFilePath);
+        }
     }
 }
 
@@ -285,12 +303,13 @@ void MainWindow::EditPrev()
     if (m_UpdatePageInProgress) return;
     int n = m_SpineList.length();
     if (n > 1) {
-        AllowSaveIfModified();
-        m_ListPtr--;
-        if (m_ListPtr < 0) m_ListPtr = n - 1;
-        m_CurrentFilePath = m_Base + m_SpineList.at(m_ListPtr);
-        ui.cbNavigate->setCurrentIndex(m_ListPtr);
-        UpdatePage(m_CurrentFilePath);
+        if (AllowSaveIfModified()) {
+            m_ListPtr--;
+            if (m_ListPtr < 0) m_ListPtr = n - 1;
+            m_CurrentFilePath = m_Base + m_SpineList.at(m_ListPtr);
+            ui.cbNavigate->setCurrentIndex(m_ListPtr);
+            UpdatePage(m_CurrentFilePath);
+        }
     }
 }
 
@@ -503,7 +522,7 @@ void MainWindow::SetupView()
     // QWebEngineView events are routed to their parent
     m_WebView->installEventFilter(this);
 
-#if !defined(Q_OS_WIN32) && !defined(Q_OS_MAC)
+#if 1 // !defined(Q_OS_WIN32) && !defined(Q_OS_MAC)
     // this may be needed by all platforms in the future
     QWidget * fp = m_WebView->focusProxy();
     if (fp) fp->installEventFilter(this);
@@ -522,6 +541,10 @@ void MainWindow::SetupView()
     m_Inspector->setObjectName("Inspector");
     addDockWidget(Qt::RightDockWidgetArea, m_Inspector);
     m_Inspector->hide();
+    m_Clips = new ClipsWindow(this);
+    m_Clips->setObjectName("ClipsWindow");
+    addDockWidget(Qt::LeftDockWidgetArea, m_Clips);
+    m_Clips->hide();
 
     // Creating the zoom controls in the status bar
     m_slZoomSlider = new QSlider(Qt::Horizontal, statusBar());
@@ -765,7 +788,6 @@ void MainWindow::UpdatePage(const QString &filename_url, const QString &source)
         }
     }
 
-
     //if isDarkMode is set, inject a local style in head
     SettingsStore settings;
     if (Utility::IsDarkMode() && settings.previewDark()) {
@@ -822,7 +844,7 @@ void MainWindow::UpdatePage(const QString &filename_url, const QString &source)
 
     // Wait until the preview is loaded before moving cursor.
     while (!m_WebView->IsLoadingFinished()) {
-        qApp->processEvents();
+        qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
     }
     
     if (!m_WebView->WasLoadOkay()) qDebug() << "WV loadFinished with okay set to false!";
@@ -830,12 +852,17 @@ void MainWindow::UpdatePage(const QString &filename_url, const QString &source)
     DBG qDebug() << "WebViewWindow UpdatePage load is Finished";
 
     UpdateWindowTitle();
-    m_source = GetSource();
     m_WebView->show();
     m_WebView->GrabFocus();
 
     // now set the mode: edit or preview
     ToggleMode(ui.actionMode->isChecked());
+    QTimer::singleShot(100, this, SLOT(SetInitialSource()));
+}
+
+void MainWindow::SetInitialSource()
+{
+    m_source = GetSource();
     m_UpdatePageInProgress = false;
 }
 
@@ -963,11 +990,12 @@ void MainWindow::LinkReturn()
     // return to last location before link
     if ((m_LastPtr != -1) && !m_LastLocation.isEmpty()) {
         if (m_LastPtr != m_ListPtr) {
-            AllowSaveIfModified();
-            m_ListPtr = m_LastPtr;
-            ui.cbNavigate->setCurrentIndex(m_ListPtr);
-            m_CurrentFilePath = m_Base + m_SpineList.at(m_ListPtr);
-            UpdatePage(m_CurrentFilePath);
+            if (AllowSaveIfModified()) {
+                m_ListPtr = m_LastPtr;
+                ui.cbNavigate->setCurrentIndex(m_ListPtr);
+                m_CurrentFilePath = m_Base + m_SpineList.at(m_ListPtr);
+                UpdatePage(m_CurrentFilePath);
+            }
         }
         ScrollTo(m_LastLocation);
     }
@@ -1007,11 +1035,12 @@ void MainWindow::LinkClicked(const QUrl &url)
             if (m_SpineList.contains(filepath)) {
                 int index = m_SpineList.indexOf(filepath);
                 if ((index > -1) && (index != m_ListPtr)) {
-                    AllowSaveIfModified();
-                    m_ListPtr = index;
-                    ui.cbNavigate->setCurrentIndex(m_ListPtr);
-                    m_CurrentFilePath = m_Base + m_SpineList.at(m_ListPtr);
-                    UpdatePage(m_CurrentFilePath);
+                    if (AllowSaveIfModified()) {
+                        m_ListPtr = index;
+                        ui.cbNavigate->setCurrentIndex(m_ListPtr);
+                        m_CurrentFilePath = m_Base + m_SpineList.at(m_ListPtr);
+                        UpdatePage(m_CurrentFilePath);
+                    }
                 }
                 if (!fragment.isEmpty()) {
                     m_WebView->ScrollToFragment(fragment);
@@ -1021,7 +1050,7 @@ void MainWindow::LinkClicked(const QUrl &url)
         }
     }
     QMessageBox::StandardButton button_pressed;
-    button_pressed = QMessageBox::warning(this, tr("PageEdit"), tr("Are you sure you want to open this link in your browser?\n\n%1").arg(toUrl.toString()), QMessageBox::Ok | QMessageBox::Cancel);
+    button_pressed = Utility::warning(this, tr("PageEdit"), tr("Are you sure you want to open this link in your browser?\n\n%1").arg(toUrl.toString()), QMessageBox::Ok | QMessageBox::Cancel);
 
     if (button_pressed == QMessageBox::Ok) {
         QDesktopServices::openUrl(toUrl);
@@ -1047,6 +1076,38 @@ void MainWindow::InspectPreviewPage()
     }
 }
 
+void MainWindow::EditClips()
+{
+    if (!m_ClipEditor->isVisible()) {
+        m_ClipEditor->show();
+        m_ClipEditor->raise();
+        m_ClipEditor->activateWindow();
+    } else {
+        m_ClipEditor->close();
+    }
+}
+
+void MainWindow::ToggleClipToolbar()
+{
+    // ui.toolBarClips->toggleViewAction();
+#if 1
+    if (!ui.toolBarClips->isVisible()) {
+        ui.toolBarClips->setVisible(true);
+    } else {
+        ui.toolBarClips->setVisible(false);
+    }
+#endif
+}
+
+void MainWindow::OpenClipsWindow()
+{
+    if (!m_Clips->isVisible()) {
+        m_Clips->show();
+    } else {
+        m_Clips->hide();
+    }
+}
+
 void MainWindow::SelectAllPreview()
 {
     m_WebView->triggerPageAction(QWebEnginePage::SelectAll);
@@ -1064,7 +1125,8 @@ void MainWindow::ReloadPreview()
     emit RequestPreviewReload();
 }
 
-void MainWindow::AllowSaveIfModified() 
+// returns false if the user wants to cancel the exit
+bool MainWindow::AllowSaveIfModified() 
 {
     // if the page source has been modified since it was loaded or saved
     // allow opportunity to save it
@@ -1081,11 +1143,12 @@ void MainWindow::AllowSaveIfModified()
     }
     if (modified) {
         QMessageBox::StandardButton button_pressed;
-        button_pressed = QMessageBox::warning(this,
+        button_pressed = Utility::warning(this,
                           tr("PageEdit"),
                           tr("Do you want to save your changes before leaving?"),
-                          QMessageBox::Save | QMessageBox::Discard
+                          QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel
                          );
+        if (button_pressed == QMessageBox::Cancel) return false;
         if (button_pressed == QMessageBox::Save) {
             bool save_result = Save();
             int cnt = 0;
@@ -1095,15 +1158,19 @@ void MainWindow::AllowSaveIfModified()
             }
         }
     }
+    return true;
 }
 
 
 void MainWindow::closeEvent(QCloseEvent *event)
 {
     ShowMessageOnStatusBar(tr("PageEdit is closing..."));
-    SaveSettings();
-    AllowSaveIfModified();
-    event->accept();
+    if (AllowSaveIfModified()) {
+        SaveSettings();
+        event->accept();
+        return;
+    }
+    event->ignore();
 }
 
 void MainWindow::moveEvent(QMoveEvent *event)
@@ -1152,50 +1219,6 @@ void MainWindow::LoadSettings()
     }
     settings.endGroup();
 
-    SettingsStore::WebViewAppearance WVAppearance = settings.webViewAppearance();
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    QWebEngineSettings *web_settings = QWebEngineSettings::defaultSettings();
-#else
-    QWebEngineSettings *web_settings = QWebEngineProfile::defaultProfile()->settings();
-#endif
-
-    // QWebEngine security settings to help prevent rogue epub3 javascripts
-    // enable javascript in mainworld for epub3 but then lock it down to the extent we can
-    web_settings->setAttribute(QWebEngineSettings::AutoLoadImages, true);
-    web_settings->setAttribute(QWebEngineSettings::JavascriptEnabled, (settings.javascriptOn() == 1));
-    web_settings->setAttribute(QWebEngineSettings::JavascriptCanOpenWindows, false);
-    web_settings->setAttribute(QWebEngineSettings::JavascriptCanAccessClipboard, false);
-    web_settings->setAttribute(QWebEngineSettings::LocalContentCanAccessRemoteUrls, (settings.remoteOn() == 1));
-    web_settings->setAttribute(QWebEngineSettings::LocalContentCanAccessFileUrls, true);
-    web_settings->setAttribute(QWebEngineSettings::PluginsEnabled, false);
-    web_settings->setAttribute(QWebEngineSettings::AutoLoadIconsForPage, false);
-    web_settings->setAttribute(QWebEngineSettings::FocusOnNavigationEnabled, false);
-    web_settings->setAttribute(QWebEngineSettings::AllowRunningInsecureContent, false);
-    web_settings->setAttribute(QWebEngineSettings::XSSAuditingEnabled, true);
-    web_settings->setAttribute(QWebEngineSettings::AllowGeolocationOnInsecureOrigins, false);
-    web_settings->setAttribute(QWebEngineSettings::ScreenCaptureEnabled, false);
-    web_settings->setAttribute(QWebEngineSettings::LocalStorageEnabled, false);
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
-    web_settings->setAttribute(QWebEngineSettings::AllowWindowActivationFromJavaScript, false);
-#endif
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 11, 0)
-    web_settings->setUnknownUrlSchemePolicy(QWebEngineSettings::DisallowUnknownUrlSchemes);
-    web_settings->setAttribute(QWebEngineSettings::PlaybackRequiresUserGesture, true);
-    web_settings->setAttribute(QWebEngineSettings::JavascriptCanPaste, false);
-#endif
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
-    web_settings->setAttribute(QWebEngineSettings::DnsPrefetchEnabled, false);
-#endif
-
-    // Our default fonts for WebView
-    web_settings->setFontSize(QWebEngineSettings::DefaultFontSize, WVAppearance.font_size);
-    web_settings->setFontFamily(QWebEngineSettings::StandardFont, WVAppearance.font_family_standard);
-    web_settings->setFontFamily(QWebEngineSettings::SerifFont, WVAppearance.font_family_serif);
-    web_settings->setFontFamily(QWebEngineSettings::SansSerifFont, WVAppearance.font_family_sans_serif);
-
     // Check for existing custom WebView stylesheet in Prefs dir
     QFileInfo CustomWebViewStylesheetInfo(QDir(Utility::DefinePrefsDir()).filePath(CUSTOM_WEBVIEW_STYLE_FILENAME));
     if (CustomWebViewStylesheetInfo.exists() &&
@@ -1696,8 +1719,8 @@ void MainWindow::SelectAll() { m_WebView->triggerPageAction(QWebEnginePage::Sele
 void MainWindow::Paste()
 {
     QClipboard *clipboard = QApplication::clipboard();
-
-    if (clipboard->mimeData()->hasHtml()) {
+    const QMimeData * mimeData = clipboard->mimeData();
+    if (mimeData->hasHtml()) {
         QMessageBox msgBox(QMessageBox::Question,
                tr("Clipboard contains HTML formatting"),
                tr("Do you want to paste clipboard data as plain text?"),
@@ -1705,20 +1728,31 @@ void MainWindow::Paste()
         msgBox.setDefaultButton(QMessageBox::Yes);
 
         // populate the detailed text window - by HTML not by the text
-        msgBox.setDetailedText(clipboard->mimeData()->html());
-
-        // show message box
-        switch (msgBox.exec()) {
-            case QMessageBox::Yes:
-                m_WebView->triggerPageAction(QWebEnginePage::PasteAndMatchStyle);
-                break;
-            case QMessageBox::No:
-                m_WebView->triggerPageAction(QWebEnginePage::Paste);
-                break;
-            default:
-                // Cancel was clicked - do nothing
-                break;
-        }
+        msgBox.setDetailedText(mimeData->html());
+        int rv = msgBox.exec();
+        if (rv == QMessageBox::Yes) {
+            m_WebView->triggerPageAction(QWebEnginePage::PasteAndMatchStyle);
+        } else if (rv == QMessageBox::No) {
+#if defined(Q_OS_WIN32)
+            // work around QtWebEngine Window's bug with clipboard html with no context
+            // by adding context back and creating a new clipboard element
+            QString html = mimeData->html();
+            QString text;
+            if (mimeData->hasText()) text = mimeData->text();
+            if (html.indexOf("<html") == -1) {
+                int pos = html.indexOf("<");
+                if (pos == -1) pos = 0;
+                html.insert(pos, "<html>\r\n<body>\r\n");
+                html.append("\r\n</body>\r\n</html>");
+                QMimeData * newdata = new QMimeData();
+                newdata->setHtml(html);
+                if (!text.isEmpty()) newdata->setText(text);
+                // ownership is passed to clipboard
+                clipboard->setMimeData(newdata);
+            }
+#endif
+            m_WebView->triggerPageAction(QWebEnginePage::Paste);
+        }  // else Cancel was clicked - do nothing
     } else {
         m_WebView->triggerPageAction(QWebEnginePage::Paste);
     }
@@ -1776,7 +1810,7 @@ void MainWindow::InsertId()
 
     // Prevent adding a hidden anchor id in PageEdit.
     if (id.isEmpty() && m_WebView->GetSelectedText().isEmpty()) {
-        QMessageBox::warning(this, tr("PageEdit"), tr("You must select text before inserting a new id."));
+        Utility::warning(this, tr("PageEdit"), tr("You must select text before inserting a new id."));
         return;
     }
 
@@ -1789,13 +1823,13 @@ void MainWindow::InsertId()
         QRegularExpressionMatch mo = invalid_id.match(selected_id);
 
         if (mo.hasMatch()) {
-            QMessageBox::warning(this, tr("PageEdit"), tr("ID is invalid - must start with a letter, followed by letter\
+            Utility::warning(this, tr("PageEdit"), tr("ID is invalid - must start with a letter, followed by letter\
  number _ : - or ."));
             return;
         };
 
         if (!m_WebView->InsertId(select_id.GetId())) {
-            QMessageBox::warning(this, tr("PageEdit"), tr("You cannot insert an id at this position."));
+            Utility::warning(this, tr("PageEdit"), tr("You cannot insert an id at this position."));
         }
     }
 }
@@ -1809,7 +1843,7 @@ void MainWindow::InsertHyperlink()
 
     // Prevent adding a hidden anchor link in PageEdit.
     if (href.isEmpty() && m_WebView->GetSelectedText().isEmpty()) {
-        QMessageBox::warning(this, tr("PageEdit"), tr("You must select text before inserting a new link."));
+        Utility::warning(this, tr("PageEdit"), tr("You must select text before inserting a new link."));
         return;
     }
 
@@ -1822,13 +1856,13 @@ void MainWindow::InsertHyperlink()
     if (select_hyperlink.exec() == QDialog::Accepted) {
         QString target = select_hyperlink.GetTarget();
         if (target.contains("<") || target.contains(">")) {
-            QMessageBox::warning(this, tr("PageEdit"), tr("Link is invalid - cannot contain '<' or '>'"));
+            Utility::warning(this, tr("PageEdit"), tr("Link is invalid - cannot contain '<' or '>'"));
             return;
         };
         // handle external links
         if (target.indexOf(':') != -1) {
             if (!m_WebView->InsertHyperlink(target)) {
-                QMessageBox::warning(this, tr("PageEdit"), tr("Error inserting external link target."));
+                Utility::warning(this, tr("PageEdit"), tr("Error inserting external link target."));
             }
         }
         // convert target to relative link from the current file
@@ -1836,7 +1870,7 @@ void MainWindow::InsertHyperlink()
         QString relative_link = Utility::buildRelativePath(m_Base + currentpath, m_Base + parts.first);
         relative_link = relative_link + parts.second;
         if (!m_WebView->InsertHyperlink(relative_link)) {
-            QMessageBox::warning(this, tr("PageEdit"), tr("Error inserting a link at this position."));
+            Utility::warning(this, tr("PageEdit"), tr("Error inserting a link at this position."));
         }
     }
 }
@@ -1891,7 +1925,7 @@ void MainWindow::InsertFiles(const QStringList &selected_files)
             html = QString("<audio controls=\"controls\" src=\"%1\">%2</audio>").arg(relative_link).arg(filename);
         }
         if (!m_WebView->InsertHtml(html)) {
-            QMessageBox::warning(this, tr("PageEdit"), tr("You cannot insert a media file at this position."));
+            Utility::warning(this, tr("PageEdit"), tr("You cannot insert a media file at this position."));
         }
     }
 }
@@ -1980,7 +2014,7 @@ bool MainWindow::MaybeSaveDialogSaysProceed(bool modified)
     if (!modified) return true;
 
     QMessageBox::StandardButton button_pressed;
-    button_pressed = QMessageBox::warning(this,
+    button_pressed = Utility::warning(this,
                     tr("PageEdit"),
                     tr("Do you want to save any changes before overwriting this file?"),
                     QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel
@@ -2010,6 +2044,63 @@ void MainWindow::AboutPageEdit()
 }
 
 
+// How to deal with this as each clipEntry struct created with new and passed via
+// emit signal to here?  Where and how should their memory be freed.
+// Perhaps we need to make clipEntry a QObject instead of just a struct or use
+// smart pointers
+void MainWindow::PasteClipEntriesIntoTarget(QList<ClipEditorModel::clipEntry *> clips)
+{
+    // target will always be the WebViewEdit    
+    if (m_WebView && m_WebView->PasteClipEntries(clips)) {
+        m_WebView->setFocus();
+        ShowMessageOnStatusBar();
+    }
+    foreach(ClipEditorModel::clipEntry * entry, clips) {
+        if (entry) delete entry;
+    }
+}
+
+void MainWindow::PasteClipNumberIntoTarget(int clip_number)
+{
+    // target will always be the WebViewEdit
+    bool applied = false;
+    if (m_WebView) {
+        applied = m_WebView->PasteClipNumber(clip_number);
+    }
+    if (applied) {
+        ShowMessageOnStatusBar(tr("Pasted clip entry %1.").arg(clip_number));
+    }
+}
+
+void MainWindow::UpdateClipButton(QAction *ui_action)
+{
+    // clipEntry is a simple struct created by GetEntry with new,
+    // no reference counting or smart pointers so they must be cleaned up appropriately
+    int clip_number = ui_action->data().toInt();
+    ClipEditorModel::clipEntry *clip_entry = ClipEditorModel::instance()->GetEntryFromNumber(clip_number);
+    if (clip_entry) {
+        ui_action->setText(clip_entry->name);
+        QString clip_text = clip_entry->text;
+        clip_text.replace('&', "&amp;").replace('<', "&lt;").replace('>', "&gt;");
+        ui_action->setToolTip(clip_text);
+        ui_action->setVisible(true);
+        // prevent memory leak
+        delete clip_entry;
+    } else {
+        ui_action->setText("");
+        ui_action->setToolTip("");
+        ui_action->setVisible(false);
+    }
+}
+
+void MainWindow::UpdateClipsUI()
+{
+    foreach(QAction * clipaction, m_clactions) {
+        UpdateClipButton(clipaction);
+    }
+}
+
+
 void MainWindow::ConnectSignalsToSlots()
 {
     MainApplication *mainApplication = qobject_cast<MainApplication *>(qApp);
@@ -2021,6 +2112,9 @@ void MainWindow::ConnectSignalsToSlots()
     connect(m_WebView,   SIGNAL(selectionChanged()),        this, SLOT(SelectionChanged()));
 
     connect(ui.actionInspect,   SIGNAL(triggered()),                       this, SLOT(InspectPreviewPage()));
+    connect(ui.actionClips,     SIGNAL(triggered()),                       this, SLOT(OpenClipsWindow()));
+    connect(ui.actionClipEdit,  SIGNAL(triggered()),                       this, SLOT(EditClips()));
+    connect(ui.actionClipBar,   SIGNAL(triggered()),                       this, SLOT(ToggleClipToolbar()));
     connect(m_SelectCharacter,  SIGNAL(SelectedCharacter(const QString&)), this, SLOT(PasteText(const QString &)));
     connect(ui.actionAbout,     SIGNAL(triggered()),                       this, SLOT(AboutPageEdit()));
 
@@ -2125,4 +2219,27 @@ void MainWindow::ConnectSignalsToSlots()
     connect(ui.actionZoomReset,  SIGNAL(triggered()),       this, SLOT(ZoomReset()));
     connect(m_slZoomSlider,      SIGNAL(valueChanged(int)), this, SLOT(SliderZoom(int)));
     connect(m_slZoomSlider,      SIGNAL(sliderMoved(int)),  this, SLOT(UpdateZoomLabel(int)));
+
+    // Clips related
+    connect(m_Clips,        SIGNAL(PasteClips(QList<ClipEditorModel::clipEntry *>)),
+            this,            SLOT(PasteClipEntriesIntoTarget(QList<ClipEditorModel::clipEntry *>)));
+    
+    connect(m_ClipEditor, SIGNAL(PasteSelectedClipRequest(QList<ClipEditorModel::clipEntry *>)),
+            this,           SLOT(PasteClipEntriesIntoTarget(QList<ClipEditorModel::clipEntry *>)));
+    
+    connect(m_ClipEditor,   SIGNAL(ShowStatusMessageRequest(const QString &)),
+            this,            SLOT(ShowMessageOnStatusBar(const QString &)));
+
+    connect(m_ClipEditor,   SIGNAL(ClipsUpdated()),
+            this,            SLOT(UpdateClipsUI()));
+    
+    foreach(QAction* clipaction, m_clactions) {
+        // Use the new signal/slot syntax and use a lambda to
+        // eliminate the need for the obsoleted QSignalMapper.
+        // [captured variables]() {...anonymous processing to do...;}
+        int i = clipaction->data().toInt();
+        connect(clipaction, &QAction::triggered, this, [this,i]() {
+                MainWindow::PasteClipNumberIntoTarget(i);
+        });
+    }
 }
